# -*- coding: utf-8 -*-
"""A dataclass that allows for an object oriented pipeline."""
import itertools
import logging
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, Optional, Tuple

import pandas as pd

import lta.helpers.data_handling as dh
import lta.helpers.jaccard as jac

logger = logging.getLogger(__name__)


@dataclass
class Pipeline:
    """The Lipid Trafficking Analysis pipeline.

    Attributes
    ----------
    file : Path
        The path to the combined data input file.
    output : Path
        Where to save the results.
    level : str
        Metadata location of experimental conditions
    tissue : str
        Metadata location of sample tissue compartment
    mode : str
        Metadata location of lipidomics mode
    thresh : float
        The fraction of samples that are 0 above which a lipid will be called 0 for a tissue.
    n : int
        Number of bootstrap replicates.
    """

    file: Path
    output: Path
    level: str
    tissue: str
    mode: str
    thresh: float
    n: int

    def __post_init__(self) -> None:
        """Post-process parameters.

        The magic of DataClasses!
        The post-init method allows for much of the processing normally required.
        Several things happen here.
        The file is read into a dataframe,
        which is split by ``Mode``,
        and stored twice as binary data (for Switch Analysis)
        and filtered counts (for ENFC Analysis).
        Any sample which contains all 0-values is dropped.
        Should the file not exist,
        not contain any data,
        or be a directory,
        then the appropriate errors are logged.
        Also, output folder is created if it does not exist already.

        Raises
        ------
        FileNotFoundError
            If self.file does not exist.
        IsADirectoryError
            If self.file is a directory.
        pd.errors.EmptyDataError
            If there is no data in self.file.
        """
        try:
            logger.debug(f"Reading data from {self.file}...")
            data = dh.construct_df(
                self.file,
                index_names=["Lipid", "Category", "m/z"],
                column_names=[
                    "Mode",
                    "Sample",
                    "Phenotype",
                    "Generation",
                    "Tissue",
                    "Handling",
                ],
                index_col=[0, 1, 2],
                header=list(range(2, 8)),
                skiprows=[8, 9, 10, 11],
            ).pipe(
                lambda x: x.loc[:, x.any()]
            )  # Drop all-0 samples
        except FileNotFoundError:
            logger.exception(f"{self.file} does not exist. A full traceback follows...")
            raise
        except IsADirectoryError:
            logger.exception(f"{self.file} is a directory. A full traceback follows...")
            raise
        except pd.errors.EmptyDataError:
            logger.exception(
                f"{self.file} contains no data. A full traceback follows..."
            )
            raise
        logger.debug("Binarizing data...")
        self.binary = {
            group: dh.not_zero(
                df,
                axis="columns",
                level=self.level,
                tissue=self.tissue,
                thresh=self.thresh,
            )
            for group, df in data.groupby(axis="columns", level=self.mode)
        }
        logger.debug("Filtering data...")
        self.filtered = {
            group: df.loc[self.binary[group].index, :]
            for group, df in data.groupby(axis="columns", level=self.mode)
        }
        self.output.mkdir(exist_ok=True, parents=True)

    def _calculate_enfc(
        self, order: Optional[Tuple[str, str]] = None
    ) -> Dict[str, pd.DataFrame]:
        """Calculate error-normalised fold change.

        Calculates the ENFC for each tissue across modes.
        For fold change to be meaningful,
        order must be specified.
        This will report fold-change as
        ``order[0] / order[1]``.

        Parameters
        ----------
        order : Tuple[str, str]
            The experimental group labels.
            logfc fill be ``order[0] / order[1]``.

        Returns
        -------
        Dict[str, pd.DataFrame]
            Key is mode, value is the ENFC data
        """
        logger.info("Calculating ENFC...")
        enfc = {
            mode: df.groupby(axis="columns", level=self.tissue).agg(
                dh.enfc,
                axis="columns",
                level=self.level,
                order=order,
            )
            for mode, df in self.filtered.items()
        }
        return enfc

    def _get_a_lipids(self) -> Dict[str, pd.DataFrame]:
        """Extract A-lipids from the dataset.

        Any tissue where more than self.thresh of the samples are 0
        is considered a total 0 for that lipid.
        Lipids that are non-0 for all tissues in any Phenotype
        are considered A-lipids.

        Returns
        -------
        Dict[str, pd.DataFrame]
            Key is mode, value is table of A-lipids
        """
        logger.info("Calculating A-lipids...")
        results = {
            f"a_{mode}": (
                df.groupby(axis="columns", level=self.level)
                .all()
                .pipe(lambda x: x.loc[x.any(axis="columns"), :])
            )
            for mode, df in self.binary.items()
        }
        return results

    def _get_b_lipids(self, picky: bool = True) -> Dict[str, pd.DataFrame]:
        """Extract B-lipids from the dataset.

        Any tissue where more than self.thresh of the samples are 0
        is considered a total 0 for that lipid.
        Lipids that are non-0 for any pair of tissues within any Phenotype
        are considered B-lipids.

        Notes
        -----
        By definition,
        all A-lipids will also be B-lipids for every pair of tissues.
        These are referred to as B-consistent,
        or Bc for short.
        Those B-lipids that are not A-lipids are also known as B-picky,
        or Bp for short.
        Which set is calculated is controlled by the boolean flag ``picky``.

        Parameters
        ----------
        picky : bool
            default=True
            If true, do **not** consider lipids that are also A-lipids

        Returns
        -------
        Dict[str, pd.DataFrame]
            Keys are the tissue pair and mode.
            Values are the table of B-lipids for that grouping.

        Raises
        ------
        AttributeError
            If A-lipids have not been previously calculated
        """
        # drop or keep a-lipids, depending on picky
        try:
            a_lip = {mode: df.index for mode, df in self.a_lipids.items()}
        except AttributeError:
            logger.exception("You must find A-lipids before B-lipids.")
            raise
        else:
            # This assumes that self.binary and a_lip will have the same keys
            # Which is definitely True
            if picky:
                data = {
                    mode: df.drop(index=a_lip[f"a_{mode}"])
                    for mode, df in self.binary.items()
                }
                subtype = "p"
            else:
                data = {
                    mode: df.loc[a_lip[f"a_{mode}"], :]
                    for mode, df in self.binary.items()
                }
                subtype = "c"

        logger.info(f"Calculating B{subtype}-lipids...")
        results = {}
        for mode, df in data.items():
            logger.debug(f"Calculating B{subtype}-lipids for {mode}...")
            tissues = df.columns.get_level_values(self.tissue)
            pairs = itertools.combinations(tissues.unique(), 2)
            for group in pairs:
                logger.debug(f"Calculating B{subtype}-lipids for {group}...")
                unified = (
                    df.loc[:, tissues.isin(group)]
                    .groupby(axis="columns", level=self.level)
                    .all()
                    .pipe(lambda x: x.loc[x.any(axis="columns"), :])
                )
                pairing = "_".join([x.upper() for x in group])
                results[f"b{subtype}_{pairing}_{mode}"] = unified
        return results

    def _get_n_lipids(self, n: int) -> Dict[str, pd.DataFrame]:
        r"""Extract N-lipids from the dataset.

        Any tissue where more than self.thresh of the samples are 0
        is considered a total 0 for that lipid.
        Lipids that are non-0 for ``n`` tissues in any Phenotype
        are considered N-lipids.

        Notes
        -----
        For historical consistency,
        N1-lipids (*ie* those found in only 1 tissue) are called U-lipids
        as they are **U**\nique.
        Also N2-lipids are not the same as B-lipids.
        A B-lipid could occur in multiple pairs of tissue,
        while N2-lipids must only occur in 1.

        Parameters
        ----------
        n : int
            The number of tissues to limit the search to

        Returns
        -------
        Dict[str, pd.DataFrame]
            Keys are the tissue group and mode.
            Values are the table of N-lipids for that grouping.
        """
        logger.info(f"Calculating N{n}-lipids...")
        results = {}
        for mode, df in self.binary.items():
            logger.debug(f"Calculating N{n}-lipids for {mode}...")
            tissues = df.columns.get_level_values(self.tissue)
            # Mask required to prevent dropping levels
            # The initial check must be done with all tissues (only n)...
            mask = (
                df.groupby(axis="columns", level=self.tissue).any().sum(axis="columns")
                == n
            )

            data = [
                (group, df.loc[mask, tissues.isin(group)])
                for group in itertools.combinations(tissues.unique(), n)
            ]
            logger.debug(
                f"N{n} tissue groups before filtering: {[group for group, _ in data]}"
            )
            # ...which necessitates a second check to drop those that are not
            # Again, mask necessary for keeping info
            # Also, we only care for groups with lipids
            masks = [
                df.groupby(axis="columns", level=self.tissue).any().sum(axis="columns")
                == n
                for _, df in data
            ]
            data = [
                (group, df.loc[mask, :].groupby(axis="columns", level=self.level).all())
                for (group, df), mask in zip(data, masks)
                if mask.sum() != 0
            ]
            logger.debug(
                f"N{n} tissue groups after filtering: {[group for group, _ in data]}"
            )
            for (tissues, df) in data:
                n_type = "u" if n == 1 else f"n{n}"
                group = "_".join([x.upper() for x in tissues])
                results[f"{n_type}_{group}_{mode}"] = df
        return results

    def _jaccard(
        self, data: Dict[str, pd.DataFrame], group: str
    ) -> Dict[str, pd.DataFrame]:
        """Calculate jaccard similarity and p-values.

        This takes a dictionary of data.
        As the output of each group of lipids will be structured as such,
        it should be called per lipid group.

        Notes
        -----
        The P-values are calculated using a bootstrap approach on a centered Jaccard similarity.

        Parameters
        ----------
        data : Dict[str, pd.DataFrame]
            A dictionary of modes and lipid data
        group : str
            Which lipids are being checke.
            This is used by logging only.

        Returns
        -------
        Dict[str, pd.DataFrame]
            Keys are the tissue group and mode.
            Values are the table of Jaccard similarity and p-values.
        """
        logger.info(f"Calculating Jaccard similarity for {group}...")
        jaccard = {
            mode: lipids.groupby(axis="index", level="Category").apply(
                lambda x: jac.bootstrap(x.iloc[:, 0], x.iloc[:, 1], self.n)
            )
            for mode, lipids in data.items()
        }
        return jaccard

    def run(self, order: Tuple[str, str]) -> None:
        """Run the full LTA pipeline.

        This:

        #. Calculates error-normalised fold change
        #. Finds A-lipids and Jaccard distances.
        #. Finds U-lipids and Jaccard distances.
        #. Finds B-lipids (both picky and consistent) and Jaccard distances.
        #. Finds N2-lipids and Jaccard distances.
        #. Writes combined results.

        Parameters
        ----------
        order : Tuple[str, str]
            The experimental group labels.
            logfc fill be ``order[0] / order[1]``.
        """
        self.enfc = self._calculate_enfc(order)

        logger.debug("Generating ENFC summary files...")
        enfc = pd.concat(self.enfc, axis="columns")
        enfc.to_csv(self.output / "enfc_individual_lipids.csv")
        enfc.groupby(axis="index", level="Category").agg(["mean", "std"]).to_csv(
            self.output / "enfc_lipid_classes.csv"
        )

        self.a_lipids = self._get_a_lipids()
        self.a_jaccard = self._jaccard(self.a_lipids, "A-lipids")

        self.u_lipids = self._get_n_lipids(1)
        self.u_jaccard = self._jaccard(self.u_lipids, "U-lipids")

        self.bc_lipids = self._get_b_lipids(picky=False)
        self.bc_jaccard = self._jaccard(self.bc_lipids, "Bc-lipids")

        self.bp_lipids = self._get_b_lipids(picky=True)
        self.bp_jaccard = self._jaccard(self.bp_lipids, "Bp-lipids")

        self.n2_lipids = self._get_n_lipids(2)
        self.n2_jaccard = self._jaccard(self.n2_lipids, "N2-lipids")

        logger.debug("Generating Switch Analysis (lipid-type) summary files...")
        summary = pd.concat(
            {
                **self.a_lipids,
                **self.bc_lipids,
                **self.bp_lipids,
                **self.n2_lipids,
                **self.u_lipids,
            },
            axis="columns",
        ).fillna(False)
        summary.columns.names = ["type_tissue_mode", "Phenotype"]
        summary.to_csv(self.output / "switch_individual_lipids.csv")
        summary.groupby(axis="index", level="Category").sum().to_csv(
            self.output / "switch_lipid_classes.csv"
        )

        logger.debug("Generating Jaccard distanse summary files...")
        jaccard = pd.concat(
            {
                **self.a_jaccard,
                **self.bc_jaccard,
                **self.bp_jaccard,
                **self.n2_jaccard,
                **self.u_jaccard,
            },
            axis="columns",
        )
        jaccard.columns.names = ["type_tissue_mode", "Metrics"]
        jaccard.to_csv(self.output / "jaccard_similarity.csv")



# -*- coding: utf-8 -*-
"""Helper functions for handling data.

In order to make the pipeline from ``lta.helpers.pipeline`` more testable,
most of its constructor and processing functions have been moved here.
This makes the code more atomic -
and, thus, testable -
by removing it from the harder to test context of the object.
"""
import logging
import sys
from pathlib import Path
from typing import Any, List, Optional, Tuple

import numpy as np
import pandas as pd

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

logger = logging.getLogger(__name__)


def construct_df(
    file: Path, index_names: List[str], column_names: List[str], **kwargs: Any
) -> pd.DataFrame:
    """Construct a dataframe from the given path.

    A light wrapper to handing basic reading and renaming of data.
    I find this function necessary as the parameters required to read metadata
    properly always seem to drop the necessary level names.
    Additionally, it also drops rows and columns that are completely NaN,
    as they are almost certainly uniformative.

    Parameters
    ----------
    file : Path
        The path to the data file
    index_names : List[str]
        The names of the index metadata columns
    column_names : List[str]
        The names of the column metadata rows
    **kwargs : Any
        Further argument passed to ``pd.read_csv``

    Returns
    -------
    pd.DataFrame
        The created dataframe
    """
    counts: pd.DataFrame = (
        pd.read_csv(file, **kwargs)
        .dropna(axis="rows", how="all")
        .dropna(axis="columns", how="all")
    )
    counts.index.names = index_names
    counts.columns.names = column_names
    return counts


def not_zero(
    df: pd.DataFrame,
    axis: Literal["index", "columns"],
    level: str,
    tissue: str,
    thresh: float,
) -> pd.DataFrame:
    """Mark any group that has more than ``thresh`` fraction 0 as 0.

    Given a counts matrix, mark all counts that are zero.
    Then, groupby the level of the specified multiindex,
    and mark a whole group as 0 if there are more than
    ``thresh * len(group)`` 0s.
    Finally,
    any lipid that is all 0 is dropped on the specified ``axis``

    Grouping occurs across tissue and level in that order.

    Parameters
    ----------
    df : pd.DataFrame
        The lipid data to convert to boolean
    axis : Literal['index', 'columns']
        Which multiindex to consider
    level : str
        The level of the multiindex to groupby
    tissue : str
        The level of the multiindex containing tissue sample
    thresh : float
        The fraction of samples above which a group is said to be 0

    Returns
    -------
    pd.DataFrame
        The processed data.
    """
    df = (
        (df == 0)
        .groupby(axis=axis, level=[tissue, level])
        .agg(lambda x: x.sum() <= (thresh * len(x)))
    )
    if axis == "index":
        df = df.loc[:, df.any(axis=axis)]
    if axis == "columns":
        df = df.loc[df.any(axis=axis), :]
    return df


def enfc(
    df: pd.DataFrame,
    axis: Literal["index", "columns"],
    level: str,
    order: Optional[Tuple[str, str]] = None,
) -> pd.DataFrame:
    """Calculate the error normalised fold change for a dataframe.

    ENFC is defined as the log foldchange of lipid levels divide by the propagated error.
    The mean and standard deviation are calculated on the groups in ``level``,
    before applying the above calculation.

    Notes
    -----
    By definition,
    fold change requires an experimental and control group,
    othewise the notion of up- or down-regulated makes no sense.
    This function assumes these groups are named "experimental" and "control",
    respectively,
    though alternatives can be passed to the ``order`` argument.

    Additionally,
    the notion of fold change requires both samples to be non-0.
    When either x/0, 0/x, or 0/0 occurs during the FC calculation,
    these values are returned as NaN.

    Parameters
    ----------
    df : pd.DataFrame
        The lipid data to convert to boolean
    axis : Literal['index', 'columns']
        Which multiindex to consider
    level : str
        The level of the multiindex containing experimental conditions
    order : Optional[Tuple[str, str]]
        default=('experimental', 'control')
        The names of the conditions to compare.
        Fold change will be ``order[0] / order[1]``.

    Returns
    -------
    pd.DataFrame
        The processed data.
    """
    if not order:
        logging.debug("Order not passed. Defaulting to ('experimental', 'control')")
        order = ("experimental", "control")
    mean = df.groupby(axis=axis, level=level).mean()
    logging.debug(f"Grouping/filtering on {axis}.")
    # Replace inf (x/0) with NaN
    # Replace 0 (0/x) with NaN
    # 0/0 is already NaN
    if axis == "index":
        logfc = np.log10(
            mean.loc[order[0], :]
            .div(mean.loc[order[1], :])
            .replace([np.inf, -np.inf, 0], np.NAN)
        )
    else:
        logfc = np.log10(
            mean.loc[:, order[0]]
            .div(mean.loc[:, order[1]])
            .replace([np.inf, -np.inf, 0], np.NAN)
        )
    error = (
        df.groupby(axis=axis, level=level).std().pow(2).sum(axis=axis).div(2).pow(0.5)
    )
    return logfc.div(error)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lta', 'lta.commands', 'lta.helpers']

package_data = \
{'': ['*']}

install_requires = \
['ConfigArgParse>=1.5.2,<2.0.0', 'numpy>=1.21.1,<2.0.0', 'pandas>=1.3.1,<2.0.0']

extras_require = \
{':python_version < "3.8"': ['typing_extensions>=3.10.0.0,<4.0.0.0']}

entry_points = \
{'console_scripts': ['lta = lta.cli:main']}

setup_kwargs = {
    'name': 'lipidta',
    'version': '0.12.3',
    'description': 'Lipid Trafficking Analysis',
    'long_description': '# Lipid Traffic Analysis\n\n[![MIT License](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)\n[![CI/CD](https://github.com/IMS-Bio2Core-Facility/lta/actions/workflows/cicd.yaml/badge.svg)](https://github.com/IMS-Bio2Core-Facility/lta/actions/workflows/cicd.yaml)\n[![codecov](https://codecov.io/gh/IMS-Bio2Core-Facility/lta/branch/main/graph/badge.svg?token=2TGYX69U3N)](https://codecov.io/gh/IMS-Bio2Core-Facility/lta)\n[![Documentation Status](https://readthedocs.org/projects/lta/badge/?version=latest)](https://lta.readthedocs.io/en/latest/?badge=latest)\n[![Project Status: Active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)\n[![Codestyle: Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n**aka LTA, aka LipidTA**\n\nA python commandline interface for analysing lipidomics data.\n\nThe source code lives on [github][github].\n\nThe documentation lives at [ReadTheDocs][readthedocs].\n\nThe project can be installed from [PyPI][pypi].\n\n## Abstract\n\nLipid Traffic Analysis (LTA) is a tool for using lipidomics data to test hypotheses about how metabolism is controlled.\nLipidomics data from several, metabolically connected tissues from control and experimental groups\ncan be used to plot the spatial or temporal distribution of lipids.\nThese distributions identify where changes in lipid metabolism occur and in which lipid pathways,\nindicating the locus and biochemical alterations that occur in a given phenotype.\nLTA was conceived in two parts.\nOne is an Abundance Analysis,\nin which the error-normalised fold change (ENFC) for the control and given phenotype group is calculated.\nBecause the ratio of the control and experimental values is scaled by the error,\nthe ENFCs are easy to plot and compare between compartments.\nThe second part is a Switch Analysis.\nThis computes the presence of variables across the network.\nCurrent development is focused on developing the technique for complex networks and on the rate of lipid transport.\n\n## Using LTA from the command line\n\n### Installation\n\n#### Installing from PyPI\n\nThis is the most straightforward way to set up the tool.\nWhen installing from PyPI,\nwe strongly reccomend using a virtual environment.\nThere are many ways to do this!\nIf you already have a preferred method -\nI use [pipx][pipx] for command line tools -\nfeel free to use that.\nOtherwise,\nuse the builtin Python module [venv][venv].\nThe exact instructions are OS-specific and detailed at the above link.\nInstructions for installing the most recent version of LTA on MacOS are given below:\n\n```shell\n# Make a directory for the project\nmkdir lta && cd lta\n# Create the virtual environment\npython3 -m venv .venv\n# Activate the environment\nsource .venv/bin/activate\n# Install lta\npip install -U LipidTA\n```\n\n```{note}\nOur pip package is `LipidTA`.\nUnfortunately,\n`lta` was "too similar to existing package names",\nso PyPi wouldn\'t let us use it.\n```\n\nIf you want to install a specific version,\nthen change the last line in the previous code block to:\n\n```shell\npip install LipidTA==0.12.1\n```\n\nreplacing the version number with the version number you want.\nA list of all released versions can be found at our [tags][tags].\n\n#### Installing from Source\n\n```{important}\nMost users **will not need** these instructions.\n```\n\nIf you need to customise the code in some manner,\nyou\'ll need to install from source.\nTo do that,\neither clone the repository from github,\nor download one of our releases.\nFor full instructions,\nplease see our guide on [contributing](./contributing.md).\n\n(data)=\n\n### The Data\n\nThis should be a single CSV files where the first 11 rows contain sample metadata\nand the first 3 columns contain the lipid metadata.\nWithin the sample metadata,\nrows 4-9 should contain the:\n\n- Mode (*ie.* -ve vs +ve)\n- Sample ID\n- Phenotype (*ie.* lean vs obese)\n- Generation (*ie.* F1 vs F2)\n- Tissue (*ie.* heart)\n- Handling (any notes about sample prep)\n\nrespectively.\nYou can name these metadata rows whatever you want,\nand tell ``lta`` where to find them with the appropriate flags.\nPlease see the section on [customising your run](customising).\nIn order to read the data,\nsome assumptions about the format must be made.\nShould we make any changes to data format expectations,\nit will be well documented and will only occur in a major/breaking releas.\n\n```{note}\nWe hope to generalise file reading in future releases to improve usability\nin a future release.\n```\n\n### Running the analysis\n\nOnce you\'ve installed the tool and activated your virtual environment,\nrunning the analysis is as simple as:\n\n```shell\nlta data.csv results\n```\n\nThe first argument is path to the combined input file.\nIf the file doesn\'t exist,\nis a directory,\nor doesn\'t contain any data files,\nthe command will error with an apropriate message.\nThe secont argument identifies a folder in which the results will be saved.\nIt will be create if it doesn\'t exist.\n\nIf you ever have any questions about the tool,\nyou can access a condensed help menu by running:\n\n```shell\nlta -h\n```\n\n(customising)=\n\n#### Customising\n\nThere are a few options that can be customised for any given run.\nThe statistics are calculated using a bootstrapping approach,\nwhich (by definition) involves repeated replicates.\nTo control the number of replicates,\npass the ``-b/--boot-reps`` flag with a number.\nGenerally, more reps improves the accuracy of the estimates,\nthough I find little improvement beyond 20,000 reps.\n1000 (the default number) seems to provide a good balance between speed and accuracy.\n\nA critical step of the analysis is binarizing the lipid expression.\nA lipid is classed as 0 in a tissue/condition if\nthe lipid is **not** detected in more than a particular fraction of samples.\nThe default values is 0.2 (one-fifth of the samples).\nIf you want to change it,\npass the ``-t/--threshold`` flag with a decimal between 0 and 1.\nThis value can have a significant impact on the analysis,\nso explore how it impacts your data!\n\nMany calculations are dependent on knowing where certain metadata is stored.\nNamely, the experimental conditions (specified with ``--phenotype``)\nthe tissue of origin (specified with ``--tissue``),\nand the lipidomics mode (specified with ``--mode``).\nIf these are not passed,\nthen they default to "Phenotype", "Tissue", and "Mode" respectively.\nPlease the section on [expected data file structure](data) for more information.\n\nFor the fold-change calculation in ENFC to make any sense,\nwe need to know which group in ``phenotype`` is which.\nYou can specify this using the ``--order`` option like so:\n\n```shell\nlta data results --order obese lean\n```\n\nThe first word following order will be treated as the experimental group,\nwhile the second word will be treated as the control group.\nIn this example then,\nfold-change would be give as ``obese / lean``.\nIf you don\'t specify,\nthis defaults to ``experimental / control``.\n\nIf you find yourself regularly passing arguments via the CLI,\nyou might want to try a configuration file!\nThis is a simple text file that stores options in a simple format:\n\n```shell\noption=value\n```\n\nBy default,\nLTA looks for ``lta_conf.txt`` in your current directory.\nHowever,\nyou can name this file whatever you want,\nand let LTA know where to find it,\nby passing the config flag like so:\n\n```shell\nlta -c path/to/your/config.txt data results\n```\n\nIf you specify an option in the configuration file,\nthat will override LTA\'s defaults,\nand specifying an option at the command line will override the configuration file!\nThe config file doesn\'t need do exist, however,\nand is just a bit of sugar.\n\n### The Output\n\n```{warning}\nRe-running the analysis overwrites existing results,\nso be sure to either back up your data,\nor pass a different output folder!\n```\n\nThe output folder will contain 5 file.\nFor each type of lipid, you should see the following:\n\n1. ``enfc_individual_lipids.csv`` - the ENFC results for each lipid.\n1. ``enfc_lipid_classes.csv`` - the mean and St.Dev. of ENFC, grouped by lipid class.\n1. ``switch_individual_lipid.csv`` - a table of lipids and their A/B/U/N classification.\n1. ``switch_lipid_classes.csv`` - a table counting the frequency of each lipid class within the A/B/U/N classification.\n1. ``jaccard_similarity.csv`` - the Jaccard similarity and p-value for each lipid class.\n\nA few notes!\nFold change will **always** be ``order[0] / order[1]``.\nThe Jaccard distances are calculated between conditions specified in ``--phenotype``\nacross both tissues and lipid classes.\nThe p-values for these distances are calculated using the method outlined by\n[N. Chung, et. al.][jaccard].\nFor ENFC,\nfold change is only meaningful if both values are non-0.\nWhere this is *not* true,\nNaN has been substituted,\nleaving an empty cell in the output.\n\n## Contributing\n\nOpen-source software is only open-source becaues of the excellent community,\nso we welcome any and all contributions!\nIf you think you have found a bug,\nplease log a report in our [issues][issues].\nIf you think you can fix a bug,\nor have an idea for a new feature,\nplease see our guide on [contributing](./contributing.md)\nfor more information on how to get started!\nWhile here,\nwe request that you follow our [code of conduct](./coc.md)\nto help maintain a welcoming,\nrespectful environment.\n\n## Future Developments\n\n- [x] Improve Github actions to use caching for poetry and Nox\n- [ ] Increase test coverage\n- [ ] Automate plotting\n\n## Citations\n\nIf you use LTA in your work,\nplease cite the following manuscripts:\n\n1. Furse, S., Watkins, A.J., Hojat, N. *et al.* Lipid Traffic Analysis reveals the impact of high paternal carbohydrate intake on offsprings’ lipid metabolism. *Commun Biol* **4**, 163 (2021). [https://doi.org/10.1038/s42003-021-01686-1][paper_1]\n1. Furse, S.[^eq], Fernandez-Twinn, D.S.[^eq], *et al.* Lipid metabolism is dysregulated before, during and after pregnancy in a mouse model of gestational diabetes. *Int. J. Mol. Sci.* **22**, 7452 (2021). [https://doi.org/10.3390/ijms22147452][paper_2]\n\n[^eq]: These authors contributed equally to this work.\n\n[github]: https://github.com/IMS-Bio2Core-Facility/lta "LTA Source Code"\n[readthedocs]: http://lta.readthedocs.io/ "LTA Documentation"\n[pypi]: https://pypi.org/project/gtexquery/ "LTA PyPI Package"\n[pipx]: https://pypa.github.io/pipx/ "pipx"\n[venv]: https://docs.python.org/3/tutorial/venv.html "Python venv"\n[tags]: https://github.com/IMS-Bio2Core-Facility/lta/releases "LTA releases"\n[issues]: https://github.com/IMS-Bio2Core-Facility/lta/issues "LTA issues"\n[jaccard]: https://doi.org/10.1186/s12859-019-3118-5 "Jaccard Probabilities"\n[paper_1]: https://www.nature.com/articles/s42003-021-01686-1 "LTA citation 1"\n[paper_2]: https://www.mdpi.com/1422-0067/22/14/7452 "LTA citation 2"\n',
    'author': 'rbpatt2019',
    'author_email': 'rb.patterson.cross@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/IMS-Bio2Core-Facility/LTA',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
